# -*- coding: cp950 -*-
##
# 類別 : spdOrderAPI
# 作者 : Allen Lee, Simon Chang
# 功能 : spdOrderAPI連接Speedy Order Server.提供對台灣交易所TWSE/OTC/TAIFEX交易的支援.

## @cond DEV
#----------------------------------------------------------------------------
import ctypes, platform, os, sys
#----------------------------------------------------------------------------
if platform.system() == 'Windows':
    print('----- spdOrderAPI detect : Windows platform -----')
    if (sys.maxsize > 2**32) == True:
        print('Load['+os.path.dirname(__file__) + '\pySpeedyAPI_64.dll'+']')
        libSpeedy = ctypes.cdll.LoadLibrary( os.path.dirname(__file__) +  '\pySpeedyAPI_64.dll')
    else:
        print('Load['+os.path.dirname(__file__) + '\pySpeedyAPI.dll'+']')
        libSpeedy = ctypes.cdll.LoadLibrary( os.path.dirname(__file__) +  '\pySpeedyAPI.dll')
elif platform.system() == 'Linux':
    print('----- spdOrderAPI detect : Linux platform -----')
    print('Load['+os.path.dirname(__file__) + '/pySpeedyAPI_64.so'+']')
    libSpeedy = ctypes.cdll.LoadLibrary(os.path.dirname(__file__) + '/pySpeedyAPI_64.so')
#----------------------------------------------------------------------------
#
# prepare for class spdOrderAPI define
#
#----------------------------------------------------------------------------
def spdOrderDummyFun():
    pass
#----------------------------------------------------------------------------
# Declare the callback type
## \hiderefby
Def_pyOrderOnConnected        = ctypes.CFUNCTYPE(None)
## \hiderefby
Def_pyOrderOnDisconnected     = ctypes.CFUNCTYPE(None)
## \hiderefby
Def_pyOrderOnLogonResponse    = ctypes.CFUNCTYPE(None, ctypes.c_bool, ctypes.c_wchar_p)
## \hiderefby
Def_pyOrderOnReplyNewOrder    = ctypes.CFUNCTYPE(None, ctypes.c_longlong, ctypes.c_wchar_p, ctypes.c_wchar_p, ctypes.c_double, ctypes.c_wchar_p
                                                     , ctypes.c_long, ctypes.c_wchar_p, ctypes.c_wchar_p, ctypes.c_wchar_p)
## \hiderefby
Def_pyOrderOnReplyCancelOrder = ctypes.CFUNCTYPE(None, ctypes.c_longlong, ctypes.c_wchar_p, ctypes.c_wchar_p, ctypes.c_double, ctypes.c_wchar_p, ctypes.c_wchar_p)
## \hiderefby
Def_pyOrderOnReplyReplaceOrder = ctypes.CFUNCTYPE(None, ctypes.c_longlong, ctypes.c_wchar_p, ctypes.c_wchar_p, ctypes.c_double, ctypes.c_wchar_p
                                                     , ctypes.c_long, ctypes.c_wchar_p, ctypes.c_wchar_p, ctypes.c_wchar_p)
## \hiderefby
Def_pyOnRejectOrder           = ctypes.CFUNCTYPE(None, ctypes.c_longlong, ctypes.c_wchar_p, ctypes.c_wchar_p, ctypes.c_wchar_p, ctypes.c_wchar_p )
## \hiderefby
Def_pyOnFill                  = ctypes.CFUNCTYPE(None, ctypes.c_longlong, ctypes.c_wchar_p, ctypes.c_wchar_p, ctypes.c_long, ctypes.c_double, ctypes.c_long, ctypes.c_wchar_p)
# Keep "life cycle" of events function pointer to prevent "access violation"
gOrderFunPtr_OnConnected        = spdOrderDummyFun
gOrderFunPtr_OnDisconnected     = spdOrderDummyFun
gOrderFunPtr_OnLogonResponse    = spdOrderDummyFun
gOrderFunPtr_OnReplyNewOrder    = spdOrderDummyFun
gOrderFunPtr_OnReplyCancelOrder = spdOrderDummyFun
gOrderFunPtr_OnReplyReplaceOrder = spdOrderDummyFun
gOrderFunPtr_OnRejectOrder      = spdOrderDummyFun
gOrderFunPtr_OnFill             = spdOrderDummyFun

## @endcond
#----------------------------------------------------------------------------
#
# class spdOrderAPI
#
#----------------------------------------------------------------------------
class spdOrderAPI():
    #------------------------------------------------------------------------
    # Functions
    #------------------------------------------------------------------------
    # 建構式中會宣告C對應在Python中的函數原形.
    # 並宣告對應C的callback function.並註冊給C的模組. 
    # @param self 物件本身 
    def __init__(self):
        # Functions definition
        # Constructor
        libSpeedy.pyOrderAdapter_New.argtypes = []
        libSpeedy.pyOrderAdapter_New.restype = ctypes.c_void_p
        # Delete
        libSpeedy.pyOrderAdapter_Delete.argtypes = [ctypes.c_void_p]
        libSpeedy.pyOrderAdapter_Delete.restype = None
        # IsConnected()
        libSpeedy.pyOrderAdapter_IsConnected.argtypes = [ctypes.c_void_p]
        libSpeedy.pyOrderAdapter_IsConnected.restype = ctypes.c_bool
        # Connect()
        libSpeedy.pyOrderAdapter_Connect.argtypes = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_int, ctypes.c_int]
        libSpeedy.pyOrderAdapter_Connect.restype = None
        #EnAbleCA
        libSpeedy.pyOrderAdapter_EnableCA.argtypes = [ctypes.c_void_p, ctypes.c_int, ctypes.c_char_p]
        libSpeedy.pyOrderAdapter_EnableCA.restype = ctypes.c_bool
        # Logon()
        libSpeedy.pyOrderAdapter_Logon.argtypes = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p]
        libSpeedy.pyOrderAdapter_Logon.restype = None
        # LogonProxy()
        libSpeedy.pyOrderAdapter_LogonProxy.argtypes = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p]
        libSpeedy.pyOrderAdapter_LogonProxy.restype = None
        # SetAccount()
        libSpeedy.pyOrderAdapter_SetAccount.argtypes = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p]
        libSpeedy.pyOrderAdapter_SetAccount.restype = None
        # Disconnect()
        libSpeedy.pyOrderAdapter_Disconnect.argtypes = [ctypes.c_void_p]
        libSpeedy.pyOrderAdapter_Disconnect.restype = None
        # SendNewOrder()
        libSpeedy.pyOrderAdapter_SendNewOrder.argtypes = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_double,
                                                          ctypes.c_char_p, ctypes.c_int, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p]
        libSpeedy.pyOrderAdapter_SendNewOrder.restype = ctypes.c_int
        # SendCancelOrder()
        libSpeedy.pyOrderAdapter_SendCancelOrder.argtypes = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p,
                                                             ctypes.c_double, ctypes.c_char_p, ctypes.c_char_p]
        libSpeedy.pyOrderAdapter_SendCancelOrder.restype = ctypes.c_int
        # SendReplaceOrder()
        libSpeedy.pyOrderAdapter_SendReplaceOrder.argtypes = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_void_p,
                                                           ctypes.c_double, ctypes.c_int, ctypes.c_char_p, ctypes.c_char_p]
        libSpeedy.pyOrderAdapter_SendReplaceOrder.restype = ctypes.c_int
        # RegEvents()
        libSpeedy.pyOrderAdapter_RegEvents.argtypes = [ctypes.c_void_p, ctypes.c_void_p, ctypes.c_void_p, ctypes.c_void_p,
                                                       ctypes.c_void_p, ctypes.c_void_p, ctypes.c_void_p, ctypes.c_void_p, ctypes.c_void_p]
        libSpeedy.pyOrderAdapter_RegEvents.restype = None

        libdir = os.path.dirname(__file__) 
        CurDir = os.path.abspath(os.getcwd())
        
        # Create pyOrderAdapter object
        os.chdir( libdir )        
        self.Obj = libSpeedy.pyOrderAdapter_New()
        os.chdir( CurDir )
        print("Package install directory: {0}".format(libdir))
        print("Current working directory: {0}".format(CurDir))

        # Keep "life cycle" of events function
        global gOrderFunPtr_OnConnected, gOrderFunPtr_OnDisconnected, gOrderFunPtr_OnLogonResponse, gOrderFunPtr_OnReplyNewOrder
        global gOrderFunPtr_OnReplyCancelOrder, gOrderFunPtr_OnReplyReplaceOrder, gOrderFunPtr_OnRejectOrder, gOrderFunPtr_OnFill
        # Binding events
        gOrderFunPtr_OnConnected         = Def_pyOrderOnConnected(self.OnConnected)
        gOrderFunPtr_OnDisconnected      = Def_pyOrderOnDisconnected(self.OnDisconnected)
        gOrderFunPtr_OnLogonResponse     = Def_pyOrderOnLogonResponse(self.OnLogonResponse)
        gOrderFunPtr_OnReplyNewOrder     = Def_pyOrderOnReplyNewOrder(self.OnReplyNewOrder)
        gOrderFunPtr_OnReplyCancelOrder  = Def_pyOrderOnReplyCancelOrder(self.OnReplyCancelOrder)
        gOrderFunPtr_OnReplyReplaceOrder = Def_pyOrderOnReplyReplaceOrder(self.OnReplyReplaceOrder)
        gOrderFunPtr_OnRejectOrder       = Def_pyOnRejectOrder(self.OnRejectOrder)
        gOrderFunPtr_OnFill              = Def_pyOnFill(self.OnFill)
        # Register events to C/C++
        libSpeedy.pyOrderAdapter_RegEvents(self.Obj, gOrderFunPtr_OnConnected, gOrderFunPtr_OnDisconnected, gOrderFunPtr_OnLogonResponse, gOrderFunPtr_OnReplyNewOrder,
                                           gOrderFunPtr_OnReplyCancelOrder, gOrderFunPtr_OnReplyReplaceOrder, gOrderFunPtr_OnRejectOrder, gOrderFunPtr_OnFill)
    
    ##連線至Speedy下單主機
    # @param self	物件本身
    # @param IP	下單主機IP
    # @param Port	下單主機Port
    # @param TimeoutSec	設定連線逾時秒數 
    def Connect(self, IP, Port, TimeoutSec):
        libSpeedy.pyOrderAdapter_Connect(self.Obj, IP.encode(), Port, TimeoutSec)
        
    ## 下單主機是否連線 
    # @param self 物件本身 
    # @return TRUE 表示連線成功,FALSE 斷線.		                                         
    def IsConnected(self):
        return libSpeedy.pyOrderAdapter_IsConnected(self.Obj)
        
    ## 使用第三方提供的簽章 API 
    # @param self 物件本身 
    # @param APICode 第三方簽章 API 代碼, 1:統一FSC ActiveX API 2:元富 MLSecurities ActiveX API  
    # @param 憑證識別資訊，通常是用戶身分證字號
    # @param self 物件本身 
    # @return TRUE 表示成功載入簽章 API,FALSE 載入失敗.		                                         
    def EnableCA(self, APICode, CommonName):
        return libSpeedy.pyOrderAdapter_EnableCA(self.Obj, APICode, CommonName.encode())
    
    ## 登入Speedy下單主機. 
    # @param self 物件本身 .
    # @param ID	Speedy登入帳號.
    # @param Password	Speedy帳號的密碼.
    # @param Account	交易帳號.
    def Logon(self, ID, Password, Account):
        libSpeedy.pyOrderAdapter_Logon(self.Obj, ID.encode(), Password.encode(), Account.encode())
    
    ## 登入Speedy Proxy下單主機 
    # @param self 物件本身 		
    # @param ID	主機帳號 
    # @param Password 主機帳號的密碼
    # @param Account交易帳號       
    #
    def LogonProxy(self, ID, Password, Account):
        libSpeedy.pyOrderAdapter_LogonProxy(self.Obj, ID.encode(), Password.encode(), Account.encode())
    
    ## 設定交易帳號 
    # @param self 物件本身 	
    # @param Exchange	交易所  ※需為大寫字母 
    #| 交易所   | 代碼     |
    #| ----:    |  :----   |
    #| 證交所   | 'TWSE'   |
    #| 櫃買中心 | 'OTC'    |
    #| 期交所   | 'TAIFEX' |
    # @param BrokerID	券商或期商代號
    # @param Account	交易帳號
    #
    def SetAccount(self, Exchange, BrokerID, Account):
        libSpeedy.pyOrderAdapter_SetAccount(self.Obj, Exchange.encode(), BrokerID.encode(), Account.encode())
        
    ## 與下單主機斷線 
    # @param self 物件本身 	
    def Disconnect(self):
        libSpeedy.pyOrderAdapter_Disconnect(self.Obj)
        
    ## 傳送新單指令    
    # @param self 物件本身.
    # @param Market	市場別  ※需為小寫字母.
    #| 市場別   | 代碼   |
    #| ----:    |  :---- |
    #| 集中市場 | 'tse'  |
    #| 櫃買市場 | 'otc'  |
    #| 期貨市場 | 'fut'  |
    #| 選擇權   | 'opt'  |
    # @param UDD 使用者運用的字串, 在以下Event 發生時會帶回, OnReplyNewOrder, OnRejectOrder, OnFill, 長度:128以內.
    # @param Symbol 商品代碼
    # @param Price 價格
    # @param Side 買賣別, ‘B’:Buy, ‘S’:Sell, ※需為大寫字母.
    # @param OrderQty 委託數量
    # @param OrderType 委託方式. ※需為大寫字母.
    #| 代碼 | 說明   |
    #| ----:|  :---- |
    #| 'L'  | Limit order, 限價單.  |
    #| 'M'  | Market order, 市價單.  |
    #| 'P'  | Market with protection, 一定範圍市價單,僅期交所支援. |
    # @param TimeInForce 委託條件 ‘R’:ROD, ‘I’:IOC, ‘F’:FOK, ※需為大寫字母.
    # @param TWSEOrdType TWSE/OTC Order type. 
    #| 代碼 | 說明   |
    #| ----:|  :---- |
    #| '0'  | regular trading Via Securities Finance.  |
    #| '1'  | Purchase on Margin.  |
    #| '2'  | Short Sell. Via Securities Firms conduct Margin Lending.  |
    #| '3'  | Purchase on Margin.  |
    #| '4'  | Short Sell.  |
    #| '5'  | SBL Short Sell type 5  |
    #| '6'  | SBL Short Sell type 5  |
    # @return NID 當日唯一的ID,收到新單回報時請透過NID對應回原單, 若NID回0表示帶入欄位有誤 
    #
    def SendNewOrder(self, Market, UDD, Symbol, Price, Side, OrderQty, OrderType, TimeInForce, TWSEOrdType ):
        return libSpeedy.pyOrderAdapter_SendNewOrder(self.Obj, Market.encode(), UDD.encode(), Symbol.encode(), Price,
                                                     Side.encode(), OrderQty, OrderType.encode(), TimeInForce.encode(), TWSEOrdType.encode())
    ## 送出刪單指令 
    # @param self	物件本身.
    # @param Market	市場別  ※需為小寫字母.
    #| 市場別   | 代碼   |
    #| ----:    |  :---- |
    #| 集中市場 | 'tse'  |
    #| 櫃買市場 | 'otc'  |
    #| 期貨市場 | 'fut'  |
    #| 選擇權   | 'opt'  |
    # @param UDD	使用者運用的字串, 在以下Event 發生時會帶回, OnReplyNewOrder, OnRejectOrder, OnFill, 長度:128以內.
    # @param Symbol	商品代碼
    # @param Price	價格
    # @param Side	買賣別, ‘B’:Buy, ‘S’:Sell, ※需為大寫字母.
    # @param OrderID	交易所端單號 
    # @return NID 當日唯一的ID,收到刪單回報時請透過NID對應回原委託單, 若NID回0表示帶入欄位有誤.
    #
    def SendCancelOrder(self, Market, UDD, Symbol, Price, Side, OrderID):
        return libSpeedy.pyOrderAdapter_SendCancelOrder(self.Obj, Market.encode(), UDD.encode(), Symbol.encode(), Price, Side.encode(), OrderID.encode())

    ## 送出改單指令 
    # @param self	物件本身.
    # @param Market	市場別  ※需為小寫字母.
    #| 市場別   | 代碼   |
    #| ----:    |  :---- |
    #| 集中市場 | 'tse'  |
    #| 櫃買市場 | 'otc'  |
    #| 期貨市場 | 'fut'  |
    #| 選擇權   | 'opt'  |
    # @param UDD	使用者運用的字串, 在以下Event 發生時會帶回, OnReplyNewOrder, OnRejectOrder, OnFill, 長度:128以內.
    # @param Symbol	商品代碼
    # @param OrderID	改單的交易所端單號
    # @param Side	買賣別, ‘B’:Buy, ‘S’:Sell, ※需為大寫字母.
    # @param Price	價格 欲改價的價錢(減量時填0)
    # @param OrderQty	委託數量 欲減的量(改價時填0)
    # @param OrderType	委託方式, ※需為大寫字母.
    #| 代碼 | 說明   |
    #| ----:|  :---- |
    #| 'L'  | Limit order, 限價單.  |
    #| 'M'  | Market order, 市價單.  |
    #| 'P'  | Market with protection, 一定範圍市價單,僅期交所支援. |
    # @param TimeInForce	委託條件 ‘R’:ROD, ‘I’:IOC, ‘F’:FOK, ※需為大寫字母.
    # @return NID 當日唯一的ID,收到改單回報時請透過NID對應回原委託單, 若NID回0表示帶入欄位有誤.
    #
    def SendReplaceOrder(self, Market, UDD, Symbol, OrderID, Side, Price, OrderQty, OrderType, TimeInForce):
        return libSpeedy.pyOrderAdapter_SendReplaceOrder(self.Obj, Market.encode(), UDD.encode(), Symbol.encode(), OrderID.encode(), Side.encode(), Price,
                                                      OrderQty, OrderType.encode(), TimeInForce.encode())        
    #------------------------------------------------------------------------
    # Events
    # inheritance spdOrderAPI & implement
    #------------------------------------------------------------------------
    ## 連線成功的 Event
    # @param self 物件本身.
    def OnConnected(self):
        pass

    ## 發生斷線的 Event
    # @param self 物件本身.
    def OnDisconnected(self):
        pass

    ## 登入結果回覆的 Event 
    # @param self  物件本身.
    # @param IsSucceed	是否成功.  True:成功 False:失敗 
    # @param ReplyString Server回復的登入訊息 
    def OnLogonResponse(self, IsSucceed, ReplyString):
        pass

    ## Event 新單回報     
    # @param self	物件本身.
    # @param NID	網路單號
    # @param UDD	SendNewOrder所帶之User Define Data
    # @param Symbol	商品代碼
    # @param Price	價格
    # @param Side	買賣別‘B’:Buy ‘S’:Sell
    # @param OrderQty	委託數量
    # @param OrderType	委託方式
    #| 代碼 | 說明   |
    #| ----:|  :---- |
    #| 'L'  | Limit order, 限價單.  |
    #| 'M'  | Market order, 市價單.  |
    #| 'P'  | Market with protection, 一定範圍市價單,僅期交所支援. |
    # @param TimeInForce	委託條件‘R’:ROD,‘I’:IOC,‘F’:FOK
    # @param OrderID	交易所端單號 
    def OnReplyNewOrder(self, NID, UDD, Symbol, Price, Side, OrderQty, OrderType, TimeInForce, OrderID):
        pass

    ## Event 刪單回報
    # @param self	物件本身.
    # @param NID	網路單號
    # @param UDD	SendCancelOrder所帶之User Define Data
    # @param Symbol	商品代碼
    # @param Price	價格
    # @param Side	買賣別‘B’:Buy ‘S’:Sell
    # @param OrderID	交易所端單號 
    def OnReplyCancelOrder(self, NID, UDD, Symbol, Price, Side, OrderID):
        pass

    ## Event 改單回報 
    # @param self	物件本身.
    # @param NID	網路單號
    # @param UDD	SendNewOrder所帶之User Define Data
    # @param Symbol	商品代碼
    # @param Price	改價後的價錢(改量時為0)
    # @param Side	買賣別‘B’:Buy ‘S’:Sell
    # @param OrderQty	改量後的量(改價時為0)
    # @param OrderType	委託方式
    #| 代碼 | 說明   |
    #| ----:|  :---- |
    #| 'L'  | Limit order, 限價單.  |
    #| 'M'  | Market order, 市價單.  |
    #| 'P'  | Market with protection, 一定範圍市價單,僅期交所支援. |
    # @param TimeInForce	委託條件‘R’:ROD,‘I’:IOC,‘F’:FOK
    # @param OrderID	交易所端單號 
    def OnReplyReplaceOrder(self, NID, UDD, Symbol, Price, Side, OrderQty, OrderType, TimeInForce, OrderID):
        pass

    ## Event  委託失敗回報 
    # @param self	物件本身.
    # @param NID	網路單號
    # @param UDD	SendNewOrder 或 SendCancelOrder 或 SendReplaceOrder 所帶之User Define Data.
    # @param ActionFrom	委託失敗類別 'N':Reject New Order,‘C’:Reject Cancel Order,‘R’: Reject Replace Order
    # @param ErrCode	錯誤代號
    # @param ErrMsg	錯誤訊息 
    def OnRejectOrder(self, NID, UDD, ActionFrom, ErrCode, ErrMsg):
        pass

    ## Event 成交回報
    # @param self	物件本身.
    # @param NID	網路單號
    # @param UDD	SendNewOrder所帶之User Define Data
    # @param OrderID	交易所端單號
    # @param ReportSequence	回報序號
    # @param FillPrice	成交價錢
    # @param FillQty	成交量
    # @param FillTime	成交時間 證交所:HHMMSSmm, 期交所:HHMMSSmmm 
    def OnFill(self, NID, UDD, OrderID, ReportSequence, FillPrice, FillQty, FillTime):
        pass
#----------------------------------------------------------------------------




from setuptools import setup, find_packages, Distribution
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open("README.md", 'r') as fh:
    long_description = fh.read()
    
class BinaryDistribution(Distribution):
    """Distribution which always forces a binary package with platform name"""
    def has_ext_modules(foo):
        return True


setup(
    name='pySpeedy',
    version='2.5.0.10',
    author='MDBS',
    author_email='simon@mdbs.com.tw',
    description='Speedy Python API',
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords='Speedy StarWave',
    packages=find_packages( exclude=['backup', 'build', 'forDoxygen']),
    package_data={'pySpeedy': ['Temp/*.*','pySpeedyAPI_64.dll','pySpeedyAPI.dll','pySpeedyAPI_64.so','spdOrderAPI.cp311-win_amd64.pyd','spdQuoteAPI.cp311-win_amd64.pyd','spdOrderAPI.cpython-36m-x86_64-linux-gnu.so','spdQuoteAPI.cpython-36m-x86_64-linux-gnu']},    
    classifiers=[
    		'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',        
    ],
    
    #distclass=BinaryDistribution
)


# -*- coding: cp950 -*-
##  @brief 
#  類別 : spdQuoteAPI
#  作者 : Allen Lee, Simon Chang
#  功能 : spdQuoteAPI 連接StarWave Marketdata Server.
#         提供對台灣交易所TWSE/OTC/TAIFEX行情的支援
#----------------------------------------------------------------------------
import ctypes, platform, os, sys
## @cond  
class spdContract(ctypes.Structure):
    _fields_ = [('BullPx', ctypes.c_double), 
                ('BearPx', ctypes.c_double), 
                ('RefPx', ctypes.c_double), 
                ('ContractMultiplier', ctypes.c_double),
                ('StrikePx', ctypes.c_double), #for Options                
                ('Market', ctypes.c_int),
                ('TradeUnit', ctypes.c_int),
                ('DayTrade', ctypes.c_int),
                ('WarningStock', ctypes.c_int),
                ('TradeFlag', ctypes.c_bool),             
                ('IsWarrant', ctypes.c_bool),             
                ('CallPut', ctypes.c_char)] #for Options
## @endcond      
class ContractInfo:
    ## 商品基本資料類別建構式.        
    def __init__(self,Ex,Sym,Name,MMD, ProdID, Info ):
        ## 交易所名稱(TWSE,OTC,TAIFEX...) 
        self.Exchange = Ex 
        ## 商品名稱.例如 證券:2330 期貨:TXFF3
        self.Symbol = Sym 
        ## 商品顯示名稱. 例如 台積電
        self.DisplayName = Name 
        ## 期貨商品結算年月 YYYYMM
        self.MaturityDate = MMD
        ## 商品類別.例如 證券:17(金融保險); 期貨:TXF 
        self.Category =  ProdID
        ## 漲停價
        self.BullPx = Info.BullPx
        ## 跌停價
        self.BearPx = Info.BearPx
        ## 參考價
        self.RefPx = Info.RefPx
        ## 契約成數.例 TXF一點 200 
        self.ContractMultiplier = Info.ContractMultiplier
        ## 選擇權商品履約價
        self.StrikePx = Info.StrikePx
        ## 市場別. 下單時使用 'fut':期貨, 'opt':選擇權, 'tse':集中市場(證交所), 'otc':櫃買市場(OTC)          
        if Info.Market ==  0:
            self.Market = 'fut'
        elif Info.Market ==  1:
            self.Market = 'opt'
        elif Info.Market ==  2:
            self.Market = 'tse'
        elif Info.Market ==  3:
            self.Market = 'otc'
        elif Info.Market ==  4:
            self.Market = 'ffut'
        elif Info.Market ==  5:
            self.Market = 'fopt'
        else:
            self.Market = 'unknown'
        ## 交易單位. 例 股票一張是1000股 
        self.TradeUnit = Info.TradeUnit
        ## 商品是否可交易 1:是 0:否
        self.TradeFlag = Info.TradeFlag        
        ## 當沖註記. 'Yes'表示為可先買後賣或先賣後買現股當沖證券，'OnlyBuy' 表示為可先買後賣現股當沖證券. 'No'表示為不可現股當沖證券。
        if Info.DayTrade  == 1:
            self.DayTrade = 'Yes'  
        elif Info.DayTrade  == 2:
            self.DayTrade = 'OnlyBuy'  
        else:
            self.DayTrade = 'No'  
        ## 是不是權證商品
        self.IsWarrant = Info.IsWarrant
        ## 警示股代碼. 0—正常 1—注意 2—處置 3—注意及處置 4—再次處置 5—注意及再次處置 6—彈性處置 7—注意及彈性處置
        self.WarringStock = Info.WarningStock  
        ## 'Call' 表示是call的選擇權,'Put'表示是Put的選擇權.'None'表示非選擇權商品        
        if Info.CallPut == 'C':
            self.CallPut = 'Call'
        elif  Info.CallPut == 'P':
            self.CallPut = 'Put'
        else:
            self.CallPut = 'None'
    ## 印出商品基本資料.   
    def Print(self):
        print( '----- [{0}][{1}] -----'.format(self.Exchange, self.Symbol))            
        print( 'Exchange[{0}] '.format(self.Exchange))            
        print( 'Symbol[{0}] '.format(self.Symbol))            
        print( 'Market[{0}] '.format(self.Market))            
        print( 'DisplayName[{0}] '.format(self.DisplayName))            
        print( 'Category[{0}] '.format(self.Category))            
        print( 'BullPx[{0}] '.format(self.BullPx))            
        print( 'BearPx[{0}] '.format(self.BearPx))            
        print( 'RefPx[{0}] '.format(self.RefPx))            
        if self.Market ==  'fut': # TAIFEX Taiwan Futures
            print( 'ContractMultiplier[{0}] '.format(self.ContractMultiplier))            
            print( 'MaturityDate[{0}] '.format(self.MaturityDate))    
            print( 'CallPut[{0}] '.format(self.CallPut))            
            print( 'StrikePx[{0}] '.format(self.StrikePx))            
            print( '----- TW Futures -----' )
        elif  self.Market ==  'opt': # TAIFEX Taiwan Options
            print( 'ContractMultiplier[{0}] '.format(self.ContractMultiplier))            
            print( 'MaturityDate[{0}] '.format(self.MaturityDate))            
            print( 'CallPut[{0}] '.format(self.CallPut))            
            print( 'StrikePx[{0}] '.format(self.StrikePx))            
            print( '----- TW Options -----' )
        elif  self.Market ==  'tse': # TWSE Taiwan equity
            print( 'TradeUnit[{0}] '.format(self.TradeUnit))            
            print( 'DayTrade[{0}] '.format(self.DayTrade))            
            print( 'WarringStock[{0}] '.format(self.WarringStock))            
            print( 'IsWarrant[{0}] '.format(self.IsWarrant))            
            print( '----- TWSE Stocks -----' )
        elif  self.Market ==  'otc': # OTC Taiwan over the counter
            print( 'TradeUnit[{0}] '.format(self.TradeUnit))            
            print( 'DayTrade[{0}] '.format(self.DayTrade))            
            print( 'WarringStock[{0}] '.format(self.WarringStock))            
            print( 'IsWarrant[{0}] '.format(self.IsWarrant))            
            print( '------ OTC Stocks -----' )
        else:
            print( 'TradeUnit[{0}] '.format(self.TradeUnit))            


#----------------------------------------------------------------------------
#
# prepare for class spdQuoteAPI define
#
#----------------------------------------------------------------------------
class spdMsgOrderBook(ctypes.Structure):
    ## 委託簿更新資料 
    # @param BidPrice1~BidPrice5	最佳買一到買五價
    # @param BidQty1~BidQty5	最佳買一到買五量
    # @param AskPrice1~AskPrice5	最佳賣一到賣五價
    # @param AskQty1~AskQty5	最佳賣一到賣五量
    # @param DerivedBidPrice	衍生Bid價格.(僅期交所提供)
    # @param DerivedBidQty	衍生Bid數量.(僅期交所提供)
    # @param DerivedAskPrice	衍生Ask價格.(僅期交所提供)
    # @param DerivedAskQty	衍生Ask數量.(僅期交所提供)
    # @param IsTestMatch    是否為試撮	
    # @param 說明	Derived Bid Ask 最佳虛擬一檔只會出現在期貨及選擇權商品， 為使未成交之ROD（Rest of day）跨月價差委託得留存於系統， 將視個別可撮合成交月份之最佳一檔委託價量，衍生虛擬委託至該跨月價差委託所屬之另一月份委託簿中。 故原個別月份除原單式委託最佳五檔買賣價量外，尚會再多揭露最佳一檔由跨月價差委託衍生之單式委託買賣價量。 該資訊由期交所提供。
    _fields_ = [('BidPrice1', ctypes.c_double), 
                ('BidPrice2', ctypes.c_double), 
                ('BidPrice3', ctypes.c_double), 
                ('BidPrice4', ctypes.c_double),
                ('BidPrice5', ctypes.c_double),
                ('BidQty1', ctypes.c_int), 
                ('BidQty2', ctypes.c_int), 
                ('BidQty3', ctypes.c_int), 
                ('BidQty4', ctypes.c_int), 
                ('BidQty5', ctypes.c_int),
                ('AskPrice1', ctypes.c_double), 
                ('AskPrice2', ctypes.c_double), 
                ('AskPrice3', ctypes.c_double), 
                ('AskPrice4', ctypes.c_double), 
                ('AskPrice5', ctypes.c_double),
                ('AskQty1', ctypes.c_int), 
                ('AskQty2', ctypes.c_int), 
                ('AskQty3', ctypes.c_int), 
                ('AskQty4', ctypes.c_int), 
                ('AskQty5', ctypes.c_int),
                ('DerivedBidPrice', ctypes.c_double),
                ('DerivedBidQty', ctypes.c_int), 
                ('DerivedAskPrice', ctypes.c_double), 
                ('DerivedAskQty', ctypes.c_int),
                ('IsTestMatch', ctypes.c_bool)]
                
## @cond                                 
#----------------------------------------------------------------------------
if platform.system() == 'Windows':
    print('\n----- spdQuoteAPI detect : Windows platform -----')
    if (sys.maxsize > 2**32) == True:
        print('Load['+os.path.dirname(__file__) + '\pySpeedyAPI_64.dll'+']')
        libSpeedy = ctypes.cdll.LoadLibrary( os.path.dirname(__file__) + '\pySpeedyAPI_64.dll')
    else:
        print('Load['+os.path.dirname(__file__) + '\pySpeedyAPI.dll'+']')
        libSpeedy = ctypes.cdll.LoadLibrary( os.path.dirname(__file__) + '\pySpeedyAPI.dll')
elif platform.system() == 'Linux':
    print('\n----- spdQuoteAPI detect : Linux platform -----')
    print('Load['+os.path.dirname(__file__) + '/pySpeedyAPI_64.so'+']')
    libSpeedy = ctypes.cdll.LoadLibrary( os.path.dirname(__file__) + '/pySpeedyAPI_64.so')
#----------------------------------------------------------------------------
def spdQuoteDummyFun():
    pass
#----------------------------------------------------------------------------
# Declare the callback type
Def_pyQuoteOnConnected     = ctypes.CFUNCTYPE(None)
Def_pyQuoteOnDisconnected  = ctypes.CFUNCTYPE(None)
Def_pyQuoteOnLogonResponse =  ctypes.CFUNCTYPE(None, ctypes.c_bool, ctypes.c_wchar_p)
Def_pyQuoteOnContract = ctypes.CFUNCTYPE(None,  ctypes.c_wchar_p, ctypes.c_wchar_p, ctypes.c_wchar_p, ctypes.c_wchar_p, ctypes.c_wchar_p,ctypes.POINTER(spdContract))
Def_pyQuoteOnContractDownloadComplete = ctypes.CFUNCTYPE(None, ctypes.c_int)
Def_pyQuoteOnOrderBook     = ctypes.CFUNCTYPE(None, ctypes.c_wchar_p, ctypes.c_wchar_p, ctypes.c_wchar_p, ctypes.POINTER(spdMsgOrderBook))
Def_pyQuoteOnTrade         = ctypes.CFUNCTYPE(None, ctypes.c_wchar_p, ctypes.c_wchar_p, ctypes.c_wchar_p, ctypes.c_double, ctypes.c_int, ctypes.c_bool)
# Keep "life cycle" of events function pointer to prevent "access violation"
gQuoteFunPtr_OnConnected     = spdQuoteDummyFun
gQuoteFunPtr_OnDisconnected  = spdQuoteDummyFun
gQuoteFunPtr_OnLogonResponse = spdQuoteDummyFun
gQuoteFunPtr_OnContract      = spdQuoteDummyFun
gQuoteFunPtr_OnComplete      = spdQuoteDummyFun
gQuoteFunPtr_OnOrderBook     = spdQuoteDummyFun
gQuoteFunPtr_OnTrade         = spdQuoteDummyFun
## @endcond
#----------------------------------------------------------------------------
#
# class spdQuoteAPI
#
#----------------------------------------------------------------------------
class spdQuoteAPI(object):    
    #------------------------------------------------------------------------
    # -- Functions --
    #------------------------------------------------------------------------
    ## 建構式中會宣告C對應在Python中的函數原形.
    #  並宣告對應C的callback function.並註冊給C的模組.     
    # @param self 物件本身 
    def __init__(self):
        # Functions definition
        # Constructor
        libSpeedy.pyQuoteAdapter_New.argtypes = []
        libSpeedy.pyQuoteAdapter_New.restype = ctypes.c_void_p
        # Delete
        libSpeedy.pyQuoteAdapter_Delete.argtypes = [ctypes.c_void_p]
        libSpeedy.pyQuoteAdapter_Delete.restype = None
        # IsConnected()
        libSpeedy.pyQuoteAdapter_IsConnected.argtypes = [ctypes.c_void_p]
        libSpeedy.pyQuoteAdapter_IsConnected.restype = ctypes.c_bool
        # Logon()
        libSpeedy.pyQuoteAdapter_Logon.argtypes = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_int, ctypes.c_char_p, ctypes.c_char_p,ctypes.c_bool]
        libSpeedy.pyQuoteAdapter_Logon.restype = None
        # Subscribe()
        libSpeedy.pyQuoteAdapter_Subscribe.argtypes = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_char_p]
        libSpeedy.pyQuoteAdapter_Subscribe.restype = ctypes.c_bool
        # DownloadContracts()
        libSpeedy.pyQuoteAdapter_DownloadContracts.argtypes = [ctypes.c_void_p]
        libSpeedy.pyQuoteAdapter_DownloadContracts.restype = None
        # RegEvents()
        libSpeedy.pyQuoteAdapter_RegEvents.argtypes = [ctypes.c_void_p, ctypes.c_void_p, ctypes.c_void_p,ctypes.c_void_p, ctypes.c_void_p, ctypes.c_void_p]
        libSpeedy.pyQuoteAdapter_RegEvents.restype = None
        
        # Create pyQuoteAdapter object        
        ## 保留原生API的Handle
        self.Obj = libSpeedy.pyQuoteAdapter_New()
        ## 股票商品基本資料Dictionary.
        self.Stocks = {}
        ## 期貨商品基本資料Dictionary.
        self.Futures = {}
        ## 選擇權商品基本資料Dictionary.
        self.Options = {}
        
        # Keep "life cycle" of events function
        global gQuoteFunPtr_OnConnected, gQuoteFunPtr_OnDisconnected, gQuoteFunPtr_OnLogonResponse, gQuoteFunPtr_OnContract, gQuoteFunPtr_OnComplete, gQuoteFunPtr_OnOrderBook, gQuoteFunPtr_OnTrade
        # Binding events
        gQuoteFunPtr_OnConnected    = Def_pyQuoteOnConnected(self.OnConnected)
        gQuoteFunPtr_OnDisconnected = Def_pyQuoteOnDisconnected(self.OnDisconnected)
        gQuoteFunPtr_OnLogonResponse= Def_pyQuoteOnLogonResponse(self.OnLogonResponse) 
        gQuoteFunPtr_OnContract     = Def_pyQuoteOnContract( self.__OnContract )    
        gQuoteFunPtr_OnComplete     = Def_pyQuoteOnContractDownloadComplete( self.__OnComplete )   
        gQuoteFunPtr_OnOrderBook    = Def_pyQuoteOnOrderBook(self.__OnOrderBook)
        gQuoteFunPtr_OnTrade        = Def_pyQuoteOnTrade(self.OnTrade)
        # Register events to C/C++
        libSpeedy.pyQuoteAdapter_RegEvents(self.Obj, gQuoteFunPtr_OnConnected, gQuoteFunPtr_OnDisconnected, gQuoteFunPtr_OnLogonResponse, gQuoteFunPtr_OnContract, gQuoteFunPtr_OnComplete, gQuoteFunPtr_OnOrderBook, gQuoteFunPtr_OnTrade)
    
    ## 行情主機是否連線 
    # @param self 物件本身 
    # @return TRUE 表示連線成功,FALSE 斷線. .		
    def IsConnected(self):
        return libSpeedy.pyQuoteAdapter_IsConnected(self.Obj)

    ## 連線登入StarWave行情主機.     
    # @param self	物件本身.
    # @param IP	行情主機IP
    # @param Port	行情主機Port
    # @param ID	StarWave登入帳號.
    # @param DownloadContracts 登入成功後是否自動下載商品基本資料.也可以登入成功後,主動呼叫DownloadContracts().
    # @param Password	StarWave帳號的密碼. 
    def Logon(self, IP, Port,ID,Password,DownloadContracts):        
        libSpeedy.pyQuoteAdapter_Logon(self.Obj, IP.encode(), Port, ID.encode(), Password.encode(), DownloadContracts )
    
    ## 訂閱商品行情 
    # 例如 訂閱台積電行情可以呼叫 ObjQuote.Subscribe( 'TWSE', '2330' )
    # @param Exchange 交易所
    #| 交易所   | 代碼     |
    #| ----:    |  :----   |
    #| 證交所   | 'TWSE'   |
    #| 櫃買中心 | 'OTC'    |
    #| 期交所   | 'TAIFEX' |
    # @param Symbol	 商品代碼
    # @return TRUE 表示訂閱成功,FALSE表示訂閱失敗.超過訂閱數上限.		
    def Subscribe(self, Exchange, Symbol):
        return libSpeedy.pyQuoteAdapter_Subscribe(self.Obj, Exchange.encode(), Symbol.encode())

    ## 用商品基本資料物件訂閱行情 
    # 例如 訂閱台積電行情可以呼叫 ObjQuote.SubscribeContract( ObjQuote.Stocks['2330'] )
    # @param Contract 商品基本資料的物件.    
    # @return TRUE 表示訂閱成功,FALSE表示訂閱失敗.超過訂閱數上限.		
    def SubscribeContract(self, Contract ):
        return libSpeedy.pyQuoteAdapter_Subscribe(self.Obj, Contract.Exchange.encode(), Contract.Symbol.encode())
     
    ## 要求下載商品基本資料
    # 當下載完成時 OnContractDownloadComplete 事件會被呼叫.   
    def DownloadContracts(self):
        return libSpeedy.pyQuoteAdapter_DownloadContracts(self.Obj )

    ## 解構式
    def __del__(self):
        pass
    #------------------------------------------------------------------------
    # -- Events --
    # inheritance spdQuoteAPI & implement
    #------------------------------------------------------------------------
## @cond                                 
    def __OnOrderBook(self, Exchange, Symbol, MsgTime, Msg):
        self.OnOrderBook(Exchange, Symbol, MsgTime, Msg[0])
        pass    

    def __OnContract(self, Ex, Sym, Name,MaturityDate, Category, Info ):        
        NewContract = ContractInfo( Ex, Sym, Name,MaturityDate, Category, Info[0] )
        if Info[0].Market ==  0:    # TAIFEX Taiwan Futures
           self.Futures[ Sym ] =  NewContract
        elif  Info[0].Market ==  1: # TAIFEX Taiwan Options
           self.Options[ Sym ] =  NewContract
        else:                       # TWSE/OTC Taiwan Stocks
           self.Stocks[ Sym ] =  NewContract
        pass

    def __OnComplete(self, Contracts ):
        print( 'Total Contract Count [{0}] '.format(Contracts))                    
        self.OnContractDownloadComplete()
        pass
## @endcond
    ## 連線完成的Event 
    # @param self 物件本身. 
    def OnConnected(self):
        pass

    ## Event 發生斷線 
    # @param self 物件本身. 
    def OnDisconnected(self):
        pass

    ## 登入結果回覆的Event 
    # @param self 物件本身. 
    # @param IsSucceed	是否成功 True:成功 False:失敗 
    # @param ReplyString 登入訊息 
    def OnLogonResponse(self, IsSucceed, ReplyString):
        pass     
    
    ## 通知商品基本資料下載完成的Event ,之後可以用Stocks,Futures,Options這幾個Dictionary取到商品基本資料
    # @param self 物件本身. 
    def OnContractDownloadComplete(self):        
        pass

    ## Event 委託簿更新
    # @param self 物件本身. 
    # @param Exchange 交易所
    #| 交易所   | 代碼     |
    #| ----:    |  :----   |
    #| 證交所   | 'TWSE'   |
    #| 櫃買中心 | 'OTC'    |
    #| 期交所   | 'TAIFEX' |
    # @param Symbol 商品代碼 
    # @param MsgTime 資料時間 格式為 HH:MM:SS.mmm 
    # @param Msg 五檔行情資料 structure為 class spdMsgOrderBook 
    def OnOrderBook(self, Exchange, Symbol, MsgTime, Msg):
        pass

    ## Event 成交信息 
    # @param self 物件本身. 
    # @param Exchange 交易所 
    #| 交易所   | 代碼     |
    #| ----:    |  :----   |
    #| 證交所   | 'TWSE'   |
    #| 櫃買中心 | 'OTC'    |
    #| 期交所   | 'TAIFEX' |
    # @param Symbol	商品代碼 		
    # @param MatchTime	成交時間 格式為 HH:MM:SS.mmm 
    # @param MatchPrice 成交價格
    # @param MatchQty	成交數量
    # @param IsTestMatch 是否為試撮
    def OnTrade(self, Exchange, Symbol, MatchTime, MatchPrice, MatchQty, IsTestMatch):
        pass
#----------------------------------------------------------------------------
